%global git_date 20230614
%global git_commit 027799d4336eb324f4543f64db8f17ad45cbcb46
%{?git_commit:%global git_commit_hash %(c=%{git_commit}; echo ${c:0:7})}


Summary:        Crypto policies used by different applications and libraries
Name:           crypto-policies
Version:        %{git_date}
Release:        8.git%{git_commit_hash}%{?dist}
License:        LGPLv2+
URL:            https://gitlab.com/redhat-crypto/fedora-crypto-policies
Source0:        https://gitlab.com/redhat-crypto/fedora-crypto-policies/-/archive/%{git_commit_hash}/%{name}-git%{git_commit_hash}.tar.gz

Patch5000:      add-SM2-and-SM2DHE-in-openssl-s-key_exchange_map.patch
Patch5001:      add-SM3-in-hash_map-of-nss.patch

BuildRequires: asciidoc
BuildRequires: make openssl java-devel bind
BuildRequires: gnutls-utils >= 3.6.0
BuildRequires: perl-interpreter perl-generators perl(File::pushd)
BuildRequires: python3-devel >= 3.6
BuildArch: noarch

Recommends: %{name}-scripts

%description
This package provides few predefined security policies such as LEGACY, DEFAULT and FUTURE
which are set system-wide by the administrator, used by various cryptographic back-ends,
such as GnuTLS, OpenSSL, NSS, BIND, libkrb5, OpenSSH, Java via OpenJDK, libssh.
Then applications that have no special needs will follow these policies by default.
That way the management of the various crypto applications and libraries used in a
system simplifies significantly.
The current implementations works by setting the desired policy in /etc/crypto-policies/config.
After this file is changed the script 'update-crypto-policies' should be executed,
and the new policies will activate.
The policies are described in a simple policy language at policies/POLICYFILE.pol,
and they operate on strings defined at the beginning of python/cryptopolicies.py.
Individual application configuration generators are present in python/policygenerators.
To generate the policies per application use the script python/build-crypto-policies.py
policydir DESTDIR or make install.

%package scripts
Summary: Tool to switch between crypto policies
Requires: %{name} = %{version}-%{release}
Recommends: grubby
Provides: fips-mode-setup = %{version}-%{release}

%description scripts
This package provides a tool update-crypto-policies, which applies
the policies provided by the crypto-policies package. These can be
either the pre-built policies from the base package or custom policies
defined in simple policy definition files.

The package also provides a tool fips-mode-setup, which can be used
to enable or disable the system FIPS mode.

%prep
%autosetup -n fedora-crypto-policies-%{git_commit_hash}-%{git_commit} -p1

%build
sed -i "s/MIN_RSA_DEFAULT = .*/MIN_RSA_DEFAULT = 'RequiredRSASize'/" \
    python/policygenerators/openssh.py
grep "MIN_RSA_DEFAULT = 'RequiredRSASize'" python/policygenerators/openssh.py

%make_build

%install
mkdir -p -m 755 %{buildroot}%{_datarootdir}/%{name}/back-ends/
mkdir -p -m 755 %{buildroot}%{_sysconfdir}/%{name}/back-ends/
mkdir -p -m 755 %{buildroot}%{_sysconfdir}/%{name}/state/
mkdir -p -m 755 %{buildroot}%{_sysconfdir}/%{name}/local.d/
mkdir -p -m 755 %{buildroot}%{_sysconfdir}/%{name}/policies/modules
mkdir -p -m 755 %{buildroot}%{_bindir}

make DESTDIR=%{buildroot} DIR=%{_datarootdir}/%{name} MANDIR=%{_mandir} %{?_smp_mflags} install
install -p -m 644 default-config %{buildroot}%{_sysconfdir}/%{name}/config
touch %{buildroot}%{_sysconfdir}/%{name}/state/current
touch %{buildroot}%{_sysconfdir}/%{name}/state/CURRENT.pol
rm -rf %{buildroot}%{_datarootdir}/%{name}/GOST-ONLY

# Create back-end configs for mounting with read-only /etc/
for d in LEGACY DEFAULT FUTURE FIPS ; do
    mkdir -p -m 755 %{buildroot}%{_datarootdir}/%{name}/back-ends/$d
    for f in %{buildroot}%{_datarootdir}/%{name}/$d/* ; do
        ln $f %{buildroot}%{_datarootdir}/%{name}/back-ends/$d/$(basename $f .txt).config
    done
done

for f in %{buildroot}%{_datarootdir}/%{name}/DEFAULT/* ; do
    ln -sf %{_datarootdir}/%{name}/DEFAULT/$(basename $f) %{buildroot}%{_sysconfdir}/%{name}/back-ends/$(basename $f .txt).config
done

%py_byte_compile %{__python3} %{buildroot}%{_datadir}/crypto-policies/python

%check
make check %{?_smp_mflags}

%post -p <lua>
if not posix.access("%{_sysconfdir}/crypto-policies/config") then
    local policy = "DEFAULT"
    local cf = io.open("/proc/sys/crypto/fips_enabled", "r")
    if cf then
        if cf:read() == "1" then
            policy = "FIPS"
        end
        cf:close()
    end
    cf = io.open("%{_sysconfdir}/crypto-policies/config", "w")
    if cf then
        cf:write(policy.."\n")
        cf:close()
    end
    cf = io.open("%{_sysconfdir}/crypto-policies/state/current", "w")
    if cf then
        cf:write(policy.."\n")
        cf:close()
    end
    local policypath = "%{_datarootdir}/crypto-policies/"..policy
    for fn in posix.files(policypath) do
        if fn ~= "." and fn ~= ".." then
            local backend = fn:gsub(".*/", ""):gsub("%%..*", "")
            local cfgfn = "%{_sysconfdir}/crypto-policies/back-ends/"..backend..".config"
            posix.unlink(cfgfn)
            posix.symlink(policypath.."/"..fn, cfgfn)
        end
    end
end

%posttrans scripts
%{_bindir}/update-crypto-policies --no-check >/dev/null 2>/dev/null || :


%files
%license COPYING.LESSER
%dir %{_sysconfdir}/crypto-policies/
%dir %{_sysconfdir}/crypto-policies/back-ends/
%dir %{_sysconfdir}/crypto-policies/state/
%dir %{_sysconfdir}/crypto-policies/local.d/
%dir %{_sysconfdir}/crypto-policies/policies/
%dir %{_sysconfdir}/crypto-policies/policies/modules/
%dir %{_datarootdir}/crypto-policies/

%ghost %config(missingok,noreplace) %{_sysconfdir}/crypto-policies/config

%ghost %config(noreplace) %{_sysconfdir}/crypto-policies/back-ends/gnutls.config
%ghost %config(noreplace) %{_sysconfdir}/crypto-policies/back-ends/openssl.config
%ghost %config(noreplace) %{_sysconfdir}/crypto-policies/back-ends/opensslcnf.config
%ghost %config(noreplace) %{_sysconfdir}/crypto-policies/back-ends/openssh.config
%ghost %config(noreplace) %{_sysconfdir}/crypto-policies/back-ends/opensshserver.config
%ghost %config(noreplace) %{_sysconfdir}/crypto-policies/back-ends/nss.config
%ghost %config(noreplace) %{_sysconfdir}/crypto-policies/back-ends/bind.config
%ghost %config(noreplace) %{_sysconfdir}/crypto-policies/back-ends/java.config
%ghost %config(noreplace) %{_sysconfdir}/crypto-policies/back-ends/javasystem.config
%ghost %config(noreplace) %{_sysconfdir}/crypto-policies/back-ends/krb5.config
%ghost %config(noreplace) %{_sysconfdir}/crypto-policies/back-ends/libreswan.config
%ghost %config(noreplace) %{_sysconfdir}/crypto-policies/back-ends/libssh.config

%ghost %{_sysconfdir}/crypto-policies/state/current
%ghost %{_sysconfdir}/crypto-policies/state/CURRENT.pol

%{_mandir}/man7/crypto-policies.7*
%{_datarootdir}/crypto-policies/LEGACY
%{_datarootdir}/crypto-policies/DEFAULT
%{_datarootdir}/crypto-policies/FUTURE
%{_datarootdir}/crypto-policies/FIPS
%{_datarootdir}/crypto-policies/EMPTY
%{_datarootdir}/crypto-policies/back-ends
%{_datarootdir}/crypto-policies/default-config
%{_datarootdir}/crypto-policies/reload-cmds.sh
%{_datarootdir}/crypto-policies/policies


%files scripts
%{_bindir}/update-crypto-policies
%{_bindir}/fips-mode-setup
%{_bindir}/fips-finish-install
%{_mandir}/man8/update-crypto-policies.8*
%{_mandir}/man8/fips-mode-setup.8*
%{_mandir}/man8/fips-finish-install.8*
%{_datarootdir}/crypto-policies/python


%changelog
* Mon Feb 10 2025 Sinong Chen <costinchen@tencent.com> - 20230614-8.git027799d
- [type] other
- [desc] add SM3 in nss.config

* Thu Sep 26 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 20230614-7.git027799d
- Rebuilt for clarifying the packages requirement in BaseOS and AppStream

* Fri Aug 16 2024 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 20230614-6.git027799d
- Rebuilt for loongarch release

* Mon May 06 2024 wynnfeng <wynnfeng@tencent.com> - 20230614-5.git027799d
- [type] other
- [desc] add SM2 and SM2DHE in openssl.config

* Mon Nov 13 2023 cunshunxia@tencent.com - 20230614-4.git027799d
- Rebuilt for bind 9.18.19

* Fri Sep 08 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 20230614-3.git027799d
- Rebuilt for OpenCloudOS Stream 23.09

* Wed Aug 23 2023 kianli@tencent.com - 20230614-2.git027799d
- Rebuilt for bind 9.18.18

* Thu Jul 20 2023 Miaojun Dong <zoedong@tencent.com> - 20230614-1.git027799d
- Bump version to 20230614

* Fri Apr 28 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 20220408-3.git97fe449
- Rebuilt for OpenCloudOS Stream 23.05

* Fri Mar 31 2023 OpenCloudOS Release Engineering <releng@opencloudos.tech> - 20220408-2.git97fe449
- Rebuilt for OpenCloudOS Stream 23

* Thu Apr 28 2022 rockerzhu <rockerzhu@tencent.com> - 20220408-1.git97fe449
- Initial build
